local MER, E, L, V, P, G = unpack(select(2, ...))
local module = MER:NewModule("mUI_databars")

--Cache global variables
--Lua functions
local _G = _G
local format, pairs, unpack = string.format, pairs, unpack
local min, mod, floor = math.min, mod, math.floor
--WoW API / Variables
local C_Timer_After = C_Timer.After
local BreakUpLargeNumbers = BreakUpLargeNumbers
local GameTooltip = GameTooltip
local GetFriendshipReputation = GetFriendshipReputation
local GetFriendshipReputationRanks = GetFriendshipReputationRanks
local GetWatchedFactionInfo = GetWatchedFactionInfo
local GetText = GetText
local GetXPExhaustion = GetXPExhaustion
local IsWatchingHonorAsXP = IsWatchingHonorAsXP
local UnitHonor = UnitHonor
local UnitHonorMax = UnitHonorMax
local UnitHonorLevel = UnitHonorLevel
local UnitLevel = UnitLevel
local UnitSex = UnitSex
local UnitXP = UnitXP
local UnitXPMax = UnitXPMax
local HONOR, LEVEL, XP, LOCKED = HONOR, LEVEL, XP, LOCKED
local TUTORIAL_TITLE26 = TUTORIAL_TITLE26
local SPELLBOOK_AVAILABLE_AT = SPELLBOOK_AVAILABLE_AT
local CreateFrame = CreateFrame
-- GLOBALS:

function module:StyleBackdrops()
	-- Experience
	local experience = _G["ElvUI_ExperienceBar"]
	if experience then
		experience:Styling()
	end

	-- Honor
	local honor = _G["ElvUI_HonorBar"]
	if honor then
		honor:Styling()
	end

	-- Reputation
	local reputation = _G["ElvUI_ReputationBar"]
	if reputation then
		reputation:Styling()
	end
end
C_Timer_After(1, module.StyleBackdrops)

local function UpdateBar(bar)
	local rest = bar.restBar
	if rest then rest:Hide() end

	if UnitLevel('player') < _G.MAX_PLAYER_LEVEL then
		local xp, mxp, rxp = UnitXP('player'), UnitXPMax('player'), GetXPExhaustion()
		bar:SetStatusBarColor(79/250, 167/250, 74/250)
		bar:SetMinMaxValues(0, mxp)
		bar:SetValue(xp)
		bar:Show()
		if rxp then
			rest:SetMinMaxValues(0, mxp)
			rest:SetValue(min(xp + rxp, mxp))
			rest:Show()
		end
	else
		bar:Hide()
	end
end

local function UpdateTooltip(bar)
	GameTooltip:SetOwner(_G.Minimap, 'ANCHOR_NONE')
	GameTooltip:SetPoint('TOPRIGHT', _G.Minimap, 'TOPLEFT', -4, -(E.db.general.minimap.size/8*E.mult)-6)
	GameTooltip:AddLine(MER.Title..L["Progress Bar"], 1, 1, 1)
	GameTooltip:AddLine(' ')

	local r, g, b = unpack(E["media"].rgbvaluecolor)

	if UnitLevel('player') < _G.MAX_PLAYER_LEVEL then
		GameTooltip:AddLine(LEVEL..' '..UnitLevel('player'), r, g, b)

		local xp, mxp, rxp = UnitXP('player'), UnitXPMax('player'), GetXPExhaustion()
		GameTooltip:AddDoubleLine(XP, xp..' / '..mxp..' ('..floor(xp/mxp*100)..'%)', 1, 1, 1, 1, 1, 1)
		if rxp then
			GameTooltip:AddDoubleLine(TUTORIAL_TITLE26, '+'..rxp..' ('..floor(rxp/mxp*100)..'%)', 1, 1, 1, 1, 1, 1)
		end
	end
	GameTooltip:Show()
end

local function OnLeave()
	GameTooltip:Hide()
end

function module:SetupScript(bar)
	bar.eventList = {
		'PLAYER_XP_UPDATE',
		'PLAYER_LEVEL_UP',
		'UPDATE_EXHAUSTION',
		'PLAYER_ENTERING_WORLD',
		'UPDATE_FACTION',
		'UNIT_INVENTORY_CHANGED',
		--'ENABLE_XP_GAIN',
		--'DISABLE_XP_GAIN',
	}
	for _, event in pairs(bar.eventList) do
		bar:RegisterEvent(event)
	end
	bar:SetScript('OnEvent', UpdateBar)
	bar:SetScript('OnEnter', UpdateTooltip)
	bar:SetScript('OnLeave', OnLeave)
end

function module:Initialize()
	module.db = E.db.mui.databars
	MER:RegisterDB(self, "databars")

	if module.db.progressbar ~= true or E.private.general.minimap.enable ~= true then return end

	local bar = CreateFrame('StatusBar', nil, _G.Minimap)
	bar:SetPoint('BOTTOM', _G.Minimap, 'TOP', 0, 1)
	bar:SetSize(E.db.general.minimap.size*E.mult, 3)
	bar:SetStatusBarTexture(E.media.normTex)
	bar:CreateBackdrop(nil, true)
	bar:SetHitRectInsets(0, 0, -10, -10)
	E:RegisterStatusBar(bar)

	local resting = CreateFrame('StatusBar', nil, bar)
	resting:SetAllPoints()
	resting:SetStatusBarTexture(E.media.normTex)
	resting:SetStatusBarColor(105/250, 194/250, 221/250, .9)
	resting:SetFrameLevel(bar:GetFrameLevel() - 1)
	bar.restBar = resting

	self:SetupScript(bar)
end

MER:RegisterModule(module:GetName())
